package gregtech.api.util;

import gregtech.api.GregTech_API;
import gregtech.api.interfaces.internal.IGT_CraftingRecipe;
import gregtech.api.items.GT_MetaGenerated_Tool;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.oredict.ShapelessOreRecipe;

public class GT_Shapeless_Recipe extends ShapelessOreRecipe implements IGT_CraftingRecipe {
	public final boolean mDismantleable, mRemovableByGT;
	private final Enchantment[] mEnchantmentsAdded;
	private final int[] mEnchantmentLevelsAdded;
	
	public GT_Shapeless_Recipe(ItemStack aResult, boolean aDismantleAble, boolean aRemovableByGT, Enchantment[] aEnchantmentsAdded, int[] aEnchantmentLevelsAdded, Object... aRecipe) {
		super(aResult, aRecipe);
		int tDamage = aResult.getItemDamage();
		mEnchantmentsAdded = aEnchantmentsAdded;
		mEnchantmentLevelsAdded = aEnchantmentLevelsAdded;
		mRemovableByGT = aRemovableByGT;
		mDismantleable = aDismantleAble || (GT_Utility.getBlockFromStack(aResult) == GregTech_API.sBlockMachines && tDamage > 0 && tDamage < GregTech_API.METATILEENTITIES.length && GregTech_API.METATILEENTITIES[tDamage] != null && GregTech_API.METATILEENTITIES[tDamage].getTileEntityBaseType() < 4);
	}
	
	@Override
	public ItemStack getCraftingResult(InventoryCrafting aGrid) {
		ItemStack rStack = super.getCraftingResult(aGrid);
		if (rStack != null) {
			// Update the Stack
			GT_Utility.updateItemStack(rStack);
			
			// Charge Values
			if (GT_ModHandler.isElectricItem(rStack)) {
				int tCharge = 0;
				for (int i = 0; i < aGrid.getSizeInventory(); i++) tCharge += GT_ModHandler.dischargeElectricItem(aGrid.getStackInSlot(i), Integer.MAX_VALUE, Integer.MAX_VALUE, true, true, true);
				if (tCharge > 0) GT_ModHandler.chargeElectricItem(rStack, tCharge, Integer.MAX_VALUE, true, false);
			}
			
			// Saving Ingredients inside the Item.
			if (mDismantleable) {
				NBTTagCompound rNBT = rStack.getTagCompound(), tNBT = new NBTTagCompound();
				if (rNBT == null) rNBT = new NBTTagCompound();
				for (int i = 0; i < 9; i++) {
					ItemStack tStack = aGrid.getStackInSlot(i);
					if (tStack != null && GT_Utility.getContainerItem(tStack, true) == null && !(tStack.getItem() instanceof GT_MetaGenerated_Tool)) {
						tStack = GT_Utility.copyAmount(1, tStack);
						GT_ModHandler.dischargeElectricItem(tStack, Integer.MAX_VALUE, Integer.MAX_VALUE, true, false, true);
						tNBT.setTag("Ingredient."+i, tStack.writeToNBT(new NBTTagCompound()));
					}
				}
				rNBT.setTag("GT.CraftingComponents", tNBT);
				rStack.setTagCompound(rNBT);
			}
			
			// Add Enchantments
			for (int i = 0; i < mEnchantmentsAdded.length; i++) GT_Utility.ItemNBT.addEnchantment(rStack, mEnchantmentsAdded[i], EnchantmentHelper.getEnchantmentLevel(mEnchantmentsAdded[i].effectId, rStack) + mEnchantmentLevelsAdded[i]);
			
			// Update the Stack again
			GT_Utility.updateItemStack(rStack);
		}
		return rStack;
	}
	
	@Override
	public boolean isRemovable() {
		return mRemovableByGT;
	}
}